% Script illustrating an application of the Lp-RLS algorithm presented in
% J. K. Pant, W.-S. Lu, and A. Antoniou, "New improved algorithms for
% compressive sensing based on Lp norm", IEEE Trans. Circuits and Systems -
% II, vol. 61, no. 3, pp. 198-202, Mar. 2014. 

N = 512;
M = 200;
K = 85;

% Construct measurement matrix
Phi = randn(M,N);
Phi = orth(Phi')'; 

% Construct a K-Sparse vector x of length N
x = zeros(N,1);
kind = randperm(N);
kind = kind(1:K);
x(kind) = randn(K,1); 

% Construct a noise vector of length M
sigma_noise = 0.01;
vec_noise = randn(M,1);
vec_noise = vec_noise - mean(vec_noise);
var_vnoise = var(vec_noise);
vec_noise = (sigma_noise/sqrt(var(vec_noise)))*vec_noise;

% Take the measurement
y = Phi*x+vec_noise;
x0 = zeros(N,1);

% Apply the Lp-RLS algorithm
p.I = 1;  p.T = 0.1;
e.I = 1; e.T = 1e-2;
lam = 7e-3;
steps = 80;
xsol = solve_LpRLS_problem(Phi,y,x0,e,p,lam,steps);

% Plot the results
figure
subplot(211)
stem(x)
axis([1 N 1.1*min(x) 1.1*max(x)])
title('Original signal')
subplot(212)
stem(xsol)
axis([1 N 1.1*min(xsol) 1.1*max(xsol)])
title('Reconstructed signal')

% Display resulting signal-to-noise ratio 
snr = 20*log10(norm(x)/norm(x-xsol));
disp([' Resulting SNR: ' num2str(snr)])